@@ -3,7 +3,7 @@ package WiringPi::API;
 use strict;
 use warnings;
 
-our $VERSION = '2.3612';
+our $VERSION = '2.3613';
 
 require XSLoader;
 XSLoader::load('WiringPi::API', $VERSION);
@@ -27,8 +27,10 @@ my @wpi_c_functions = qw(
     pseudoPinsSetup     wiringPiSPISetup    spiDataRW
     wiringPiI2CSetup    wiringPiI2CSetupInterface
     wiringPiI2CRead     wiringPiI2CReadReg8 wiringPiI2CReadReg16
-    wiringPiI2CWrite    wiringPiI2CWriteReg8    wiringPiI2CWriteReg16
-    pinModeAlt
+    wiringPiI2CWrite    wiringPiI2CWriteReg8 wiringPiI2CWriteReg16
+    pinModeAlt          serialOpen          serialFlush
+    serialPutchar       serialPuts          serialDataAvail
+    serialGetchar
 );
 
 my @wpi_perl_functions = qw(
@@ -43,7 +45,9 @@ my @wpi_perl_functions = qw(
     ads1115_setup   spi_setup       spi_data            i2c_setup
     i2c_interface   i2c_read        i2c_read_byte       i2c_read_word
     i2c_write       i2c_write_byte  i2c_write_word      testChar
-    phys_to_wpi     pin_mode_alt
+    phys_to_wpi     pin_mode_alt    serial_open         serial_flush
+    serial_put_char serial_puts     serial_data_avail   serial_get_char 
+    serial_close    serial_gets
 );
 
 our @EXPORT_OK;
@@ -59,6 +63,54 @@ sub new {
     return bless {}, shift;
 }
 
+# serial functions
+
+sub serial_open {
+    shift if @_ > 2;
+    my ($dev_ptr, $baud) = @_;
+    my $fd = serialOpen($dev_ptr, $baud);
+    die "could not open serial device $dev_ptr\n" if $fd == -1;
+    return $fd;
+}
+sub serial_close {
+    shift if @_ > 1;
+    my ($fd) = @_;
+    serialClose($fd);
+}
+sub serial_flush {
+    shift if @_ > 1;
+    my ($fd) = @_;
+    serialFlush($fd);
+}
+sub serial_put_char {
+    shift if @_ > 2;
+    my ($fd, $unsigned_char) = @_;
+    serialPutchar($fd, $unsigned_char);
+}
+sub serial_puts {
+    shift if @_ > 2;
+    my ($fd, $char) = @_;
+    serialPuts($fd, $char);
+}
+sub serial_data_avail {
+    shift if @_ > 1;
+    my ($fd) = @_;
+    serialDataAvail($fd);
+}
+sub serial_get_char {
+    shift if @_ > 1;
+    my ($fd) = @_;
+    serialGetchar($fd);
+}
+sub serial_gets {
+    shift if @_ > 2;
+    my ($fd, $nbytes) = @_;
+    my $buf = "";
+    my $char_ptr = serialGets($fd, $buf, $nbytes);
+    my $unpacked = unpack "A*", $char_ptr;
+    return $unpacked;
+}
+
 # interrupt functions
 
 sub set_interrupt {
@@ -509,6 +561,8 @@ versions, but are still 100% compatible.
     set_interrupt   pin_mode        analog_read         analog_write
     shift_reg_setup bmp180_setup    bmp180_pressure     bmp180_temp
     ads1115_setup   spi_setup       spi_data            phys_to_wpi
+    serial_open     serial_flush    serial_put_char     serial_puts
+    serial_get_char serial_close    serial_data_avail
 
 =head1 EXPORT_TAGS
 
@@ -542,7 +596,11 @@ See L</ADC FUNCTIONS>.
 
 =head2 SHIFT REGISTER
 
-See L</SHIFT REGISTER FUNCTIONS>.
+See L</SHIFT REGISTER FUNCTIONS>
+
+=head2 SERIAL
+
+See L</SERIAL FUNCTIONS>
 
 =head2 I2C
 
@@ -858,6 +916,12 @@ Note: When in 4-bit mode, the C<d0> through C<3> parameters actually map to
 pins C<d4> through C<d7> on the LCD board, so you need to connect those pins
 to their respective selected GPIO pins.
 
+NOTE: There is an upper limit of the number of LCDs that can be initialized
+simultaneously. This number is 8 (0-7). Always check the return of this
+function to ensure you're under the maximum file descriptors. If you receive a
+`-1`, you're out of bounds, and any functions called on the LCD will cause a 
+segmentation fault.
+
 =head2 lcd_home($fd)
 
 Maps to C<void lcdHome(int fd)>
@@ -1146,6 +1210,108 @@ Mandatory: Integer, the GPIO pin number connected to the register's C<SHCP> pin
 Mandatory: Integer, the GPIO pin number connected to the register's C<STCP> pin
 (12). Can be any GPIO pin capable of output.
 
+=head1 SERIAL FUNCTIONS
+
+These functions provide basic access to read and write to a serial device.
+
+=head2 serial_open($device, $baud)
+
+Maps to C<int serialOpen(const char *device, const int baud)>
+
+Opens a serial device for read/write access.
+
+Parameters:
+
+    $device
+
+Mandatory, String: The name of the serial device, eg: C</dev/ttyACM0>.
+
+    $baud
+
+Mandatory, Integer: The speed of the serial device. (eg: C<9600>).
+
+Return, Integer: The file descriptor of the device.
+
+=head2 serial_close($fd)
+
+Maps to C<void serialClose(const int fd)>
+
+Closes an already open serial device.
+
+Parameters:
+
+    $fd
+
+Mandatory, Integer: The file descriptor returned by your call to C<serial_open()>.
+
+=head2 serial_flush($fd)
+
+Maps to C<serialFlush(const int fd)>
+
+Flushes the serial device's buffer.
+
+Parameters:
+
+    $fd
+
+Mandatory, Integer: The file descriptor returned by your call to C<serial_open()>.
+
+=head2 serial_data_avail($fd)
+
+Maps to C<serialDataAvail(const int fd)>
+
+Check if there is any data available on the serial interface.
+
+Parameters:
+
+    $fd
+
+Mandatory, Integer: The file descriptor returned by your call to C<serial_open()>.
+
+=head2 serial_get_char($fd)
+
+Maps to C<serialGetchar(const int fd)>
+
+Read a single byte from the serial interface.
+
+Parameters:
+
+    $fd
+
+Mandatory, Integer: The file descriptor returned by your call to C<serial_open()>.
+
+=head2 serial_put_char($fd, $char)
+
+Maps to C<serialPutchar(const int fd, const unsigned char c)>
+
+Write a single byte to the interface.
+
+Parameters:
+
+    $fd
+
+Mandatory, Integer: The file descriptor returned by your call to C<serial_open()>.
+
+    $char
+
+Mandatory, Byte: A single byte to write to the serial interface.
+
+=head2 serial_puts($fd, $string)
+
+Maps to C<serialPuts(const int fd, const char* string)>
+
+Write an arbitrary length string to the serial interface.
+
+Parameters:
+
+    $fd
+
+Mandatory, Integer: The file descriptor returned by your call to C<serial_open()>.
+
+    $string
+
+Mandatory, String: The content to write to the device.
+
 =head1 I2C FUNCTIONS
 
 These functions allow you to read and write devices on the Inter-Integrated
